@@ -126,18 +126,13 @@ bool K3bCdrecordProgram::scan( const QString& p )
     return false;
 
   bool wodim = false;
-  bool cdrskin = false;
   QString path = p;
   QFileInfo fi( path );
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
 
-    if( QFile::exists( path + "cdrskin" ) ) {
-      cdrskin = true;
-      path += "cdrskin";
-    }
-    else if( QFile::exists( path + "wodim" ) ) {
+    if( QFile::exists( path + "wodim" ) ) {
       wodim = true;
       path += "wodim";
     }
@@ -707,7 +702,7 @@ bool K3bVcdbuilderProgram::scan( const QString& p )
 
 
 K3bNormalizeProgram::K3bNormalizeProgram()
-  : K3bExternalProgram( "normalize-audio" )
+  : K3bExternalProgram( "normalize" )
 {
 }
 
@@ -722,7 +717,7 @@ bool K3bNormalizeProgram::scan( const QString& p )
   if( fi.isDir() ) {
     if( path[path.length()-1] != '/' )
       path.append("/");
-    path.append("normalize-audio");
+    path.append("normalize");
   }
 
   if( !QFile::exists( path ) )
@@ -862,7 +857,7 @@ bool K3bDvdformatProgram::scan( const QString& p )
   if( vp.start( KProcess::Block, KProcess::AllOutput ) ) {
     // different locales make searching for the +- char difficult
     // so we simply ignore it.
-    int pos = out.output().find( QRegExp("DVD.*RAM format utility") );
+    int pos = out.output().find( QRegExp("DVD.*RW(/-RAM)? format utility") );
     if( pos < 0 )
       return false;
 
