@@ -1,10 +1,10 @@
-/* 
+/*
  *
- * $Id: k3bgrowisofswriter.cpp 512487 2006-02-22 16:58:18Z trueg $
+ * $Id: k3bgrowisofswriter.cpp 679276 2007-06-23 13:25:21Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "k3bgrowisofshandler.h"
 #include <k3bpipebuffer.h>
 #include <k3bglobalsettings.h>
+#include <k3bdeviceglobals.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -41,25 +42,29 @@
 class K3bGrowisofsWriter::Private
 {
 public:
-  Private() 
+  Private()
     : writingMode( 0 ),
       closeDvd(false),
+      multiSession(false),
       process( 0 ),
       growisofsBin( 0 ),
       trackSize(-1),
       layerBreak(0),
       usingRingBuffer(false),
-      ringBuffer(0) {
+      ringBuffer(0),
+    forceNoEject( false ) {
   }
 
   int writingMode;
   bool closeDvd;
+  bool multiSession;
   K3bProcess* process;
   const K3bExternalBin* growisofsBin;
   QString image;
 
   bool success;
   bool canceled;
+  bool finished;
 
   QTime lastSpeedCalculationTime;
   int lastSpeedCalculationBytes;
@@ -78,11 +83,16 @@ public:
   long layerBreak;
 
   unsigned long long overallSizeFromOutput;
+  long long firstSizeFromOutput;
 
   QFile inputFile;
 
   bool usingRingBuffer;
   K3bPipeBuffer* ringBuffer;
+
+  QString multiSessionInfo;
+
+    bool forceNoEject;
 };
 
 
@@ -148,10 +158,10 @@ bool K3bGrowisofsWriter::prepareProcess()
     emit infoMessage( i18n("Could not find %1 executable.").arg("growisofs"), ERROR );
     return false;
   }
-  
+
   if( d->growisofsBin->version < K3bVersion( 5, 10 ) ) {
     emit infoMessage( i18n("Growisofs version %1 is too old. "
-			   "K3b needs at least version 5.10.").arg(d->growisofsBin->version), 
+			   "K3b needs at least version 5.10.").arg(d->growisofsBin->version),
 		      ERROR );
     return false;
   }
@@ -185,6 +195,9 @@ bool K3bGrowisofsWriter::prepareProcess()
   // to write zeros. We just tell growisofs to reserve a multiple of 16 blocks.
   // This is only releveant in DAO mode anyway.
   //
+  // FIXME: seems as we also need this for double layer writing. Better make it the default and
+  //        actually write the pad bytes. The only possibility I see right now is to add a padding option
+  //        to the pipebuffer.
   int trackSizePadding = 0;
   if( d->trackSize > 0 && d->growisofsBin->version < K3bVersion( 5, 20 ) ) {
     if( d->trackSize % 16 ) {
@@ -207,8 +220,14 @@ bool K3bGrowisofsWriter::prepareProcess()
   else
     s += d->image;
 
-  // for now we do not support multisession
-  *d->process << "-Z" << s;
+  if( d->multiSession && !d->multiSessionInfo.isEmpty() )
+    *d->process << "-C" << d->multiSessionInfo;
+
+  if( d->multiSession )
+    *d->process << "-M";
+  else
+    *d->process << "-Z";
+  *d->process << s;
 
 
   if( !d->image.isEmpty() && d->usingRingBuffer ) {
@@ -237,7 +256,7 @@ bool K3bGrowisofsWriter::prepareProcess()
   // the tracksize parameter takes priority over the dao:tracksize parameter since growisofs 5.18
   else if( d->growisofsBin->version > K3bVersion( 5, 17 ) && d->trackSize > 0 )
     *d->process << "-use-the-force-luke=tracksize:" + QString::number(d->trackSize + trackSizePadding);
-  
+
   if( simulate() )
     *d->process << "-use-the-force-luke=dummy";
 
@@ -252,7 +271,12 @@ bool K3bGrowisofsWriter::prepareProcess()
   else
     d->gh->reset( burnDevice(), false );
 
-  if( dvdCompat )
+  //
+  // Never use the -dvd-compat parameter with DVD+RW media
+  // because the only thing it does is creating problems.
+  // Normally this should be done in growisofs
+  //
+  if( dvdCompat && burnDevice()->mediaType() != K3bDevice::MEDIA_DVD_PLUS_RW )
     *d->process << "-dvd-compat";
 
   //
@@ -270,7 +294,7 @@ bool K3bGrowisofsWriter::prepareProcess()
       speed = burnDevice()->determineMaximalWriteSpeed();
     }
 
-    // speed may be a float number. example: DVD+R(W): 2.4x    
+    // speed may be a float number. example: DVD+R(W): 2.4x
     if( speed != 0 )
       *d->process << QString("-speed=%1").arg( speed%1385 > 0
 					      ? QString::number( (float)speed/1385.0, 'f', 1 )
@@ -291,25 +315,29 @@ bool K3bGrowisofsWriter::prepareProcess()
   for( QStringList::const_iterator it = params.begin(); it != params.end(); ++it )
     *d->process << *it;
 
+  emit debuggingOutput( "Burned media", K3bDevice::mediaTypeString(burnDevice()->mediaType()) );
+
   return true;
 }
 
 
 void K3bGrowisofsWriter::start()
 {
-  emit started();
+  jobStarted();
 
   d->lastWritingSpeed = 0;
   d->lastProgressed = 0;
   d->lastProgress = 0;
+  d->firstSizeFromOutput = -1;
   d->lastSpeedCalculationTime = QTime::currentTime();
   d->lastSpeedCalculationBytes = 0;
   d->writingStarted = false;
   d->canceled = false;
   d->speedEst->reset();
+  d->finished = false;
 
   if( !prepareProcess() ) {
-    emit finished( false );
+    jobFinished( false );
   }
   else {
 
@@ -325,22 +353,36 @@ void K3bGrowisofsWriter::start()
 
     emit newSubTask( i18n("Preparing write process...") );
 
+    // FIXME: check the return value
+    if( K3b::isMounted( burnDevice() ) ) {
+      emit infoMessage( i18n("Unmounting medium"), INFO );
+      K3b::unmount( burnDevice() );
+    }
+
+    // block the device (including certain checks)
+    k3bcore->blockDevice( burnDevice() );
+
+    // lock the device for good in this process since it will
+    // be opened in the growisofs process
+    burnDevice()->close();
+    burnDevice()->usageLock();
+
     if( !d->process->start( KProcess::NotifyOnExit, KProcess::All ) ) {
       // something went wrong when starting the program
       // it "should" be the executable
       kdDebug() << "(K3bGrowisofsWriter) could not start " << d->growisofsBin->path << endl;
       emit infoMessage( i18n("Could not start %1.").arg(d->growisofsBin->name()), K3bJob::ERROR );
-      emit finished(false);
+      jobFinished(false);
     }
     else {
       if( simulate() ) {
 	emit newTask( i18n("Simulating") );
-	emit infoMessage( i18n("Starting simulation..."), 
+	emit infoMessage( i18n("Starting simulation..."),
 			  K3bJob::INFO );
       }
       else {
 	emit newTask( i18n("Writing") );
-	emit infoMessage( i18n("Starting writing..."), K3bJob::INFO );
+	emit infoMessage( i18n("Starting disc write..."), K3bJob::INFO );
       }
 
       d->gh->handleStart();
@@ -355,7 +397,7 @@ void K3bGrowisofsWriter::start()
 
 	d->ringBuffer->writeToFd( d->process->stdinFd() );
 	bool manualBufferSize = k3bcore->globalSettings()->useManualBufferSize();
-	int bufSize = ( manualBufferSize ? k3bcore->globalSettings()->bufferSize() : 4 );
+	int bufSize = ( manualBufferSize ? k3bcore->globalSettings()->bufferSize() : 20 );
 	d->ringBuffer->setBufferSize( bufSize );
 
 	if( !d->image.isEmpty() )
@@ -404,6 +446,12 @@ void K3bGrowisofsWriter::setCloseDvd( bool b )
 }
 
 
+void K3bGrowisofsWriter::setMultiSession( bool b )
+{
+  d->multiSession = b;
+}
+
+
 void K3bGrowisofsWriter::setImageToWrite( const QString& filename )
 {
   d->image = filename;
@@ -420,21 +468,27 @@ void K3bGrowisofsWriter::slotReceivedStderr( const QString& line )
       d->writingStarted = true;
       emit newSubTask( i18n("Writing data") );
     }
-    
+
     // parse progress
     int pos = line.find( "/" );
     unsigned long long done = line.left( pos ).toULongLong();
     bool ok = true;
     d->overallSizeFromOutput = line.mid( pos+1, line.find( "(", pos ) - pos - 1 ).toULongLong( &ok );
+    if( d->firstSizeFromOutput == -1 )
+      d->firstSizeFromOutput = done;
+    done -= d->firstSizeFromOutput;
+    d->overallSizeFromOutput -= d->firstSizeFromOutput;
     if( ok ) {
       int p = (int)(100 * done / d->overallSizeFromOutput);
       if( p > d->lastProgress ) {
 	emit percent( p );
+	emit subPercent( p );
 	d->lastProgress = p;
       }
       if( (unsigned int)(done/1024/1024) > d->lastProgressed ) {
 	d->lastProgressed = (unsigned int)(done/1024/1024);
 	emit processedSize( d->lastProgressed, (int)(d->overallSizeFromOutput/1024/1024)  );
+	emit processedSubSize( d->lastProgressed, (int)(d->overallSizeFromOutput/1024/1024)  );
       }
 
       // try parsing write speed (since growisofs 5.11)
@@ -448,7 +502,7 @@ void K3bGrowisofsWriter::slotReceivedStderr( const QString& line )
 	  d->lastWritingSpeed = speed;
 	}
 	else
-	  kdDebug() << "(K3bGrowisofsWriter) speed parsing failed: '" 
+	  kdDebug() << "(K3bGrowisofsWriter) speed parsing failed: '"
 		    << line.mid( pos, line.find( 'x', pos ) - pos ) << "'" << endl;
       }
       else {
@@ -456,11 +510,14 @@ void K3bGrowisofsWriter::slotReceivedStderr( const QString& line )
       }
     }
     else
-      kdDebug() << "(K3bGrowisofsWriter) progress parsing failed: '" 
+      kdDebug() << "(K3bGrowisofsWriter) progress parsing failed: '"
 		<< line.mid( pos+1, line.find( "(", pos ) - pos - 1 ).stripWhiteSpace() << "'" << endl;
   }
 
-  d->gh->handleLine( line );
+  //  else
+  // to be able to parse the ring buffer fill in growisofs 6.0 we need to do this all the time
+  // FIXME: get rid of the K3bGrowisofsHandler once it is sure that we do not need the K3bGrowisofsImager anymore
+    d->gh->handleLine( line );
 }
 
 
@@ -468,24 +525,34 @@ void K3bGrowisofsWriter::slotProcessExited( KProcess* p )
 {
   d->inputFile.close();
 
+  // release the device within this process
+  burnDevice()->usageUnlock();
+
+  // unblock the device
+  k3bcore->unblockDevice( burnDevice() );
+
   if( d->canceled ) {
-    if( !d->usingRingBuffer || !d->ringBuffer->active() ) {
+    if( !d->finished ) {
+      d->finished = true;
       // this will unblock and eject the drive and emit the finished/canceled signals
       K3bAbstractWriter::cancel();
     }
     return;
   }
 
-  if( p->exitStatus() == 0 ) {
+  d->finished = true;
+
+  // it seems that growisofs sometimes exits with a valid exit code while a write error occured
+  if( p->exitStatus() == 0 && d->gh->error() != K3bGrowisofsHandler::ERROR_WRITE_FAILED ) {
 
     int s = d->speedEst->average();
     if( s > 0 )
       emit infoMessage( i18n("Average overall write speed: %1 KB/s (%2x)").arg(s).arg(KGlobal::locale()->formatNumber((double)s/1385.0), 2), INFO );
 
     if( simulate() )
-      emit infoMessage( i18n("Simulation successfully finished"), K3bJob::SUCCESS );
+      emit infoMessage( i18n("Simulation successfully completed"), K3bJob::SUCCESS );
     else
-      emit infoMessage( i18n("Writing successfully finished"), K3bJob::SUCCESS );
+      emit infoMessage( i18n("Writing successfully completed"), K3bJob::SUCCESS );
 
     d->success = true;
   }
@@ -495,13 +562,13 @@ void K3bGrowisofsWriter::slotProcessExited( KProcess* p )
     d->success = false;
   }
 
-  if( !k3bcore->globalSettings()->ejectMedia() )
-    emit finished(d->success);
+  if( !k3bcore->globalSettings()->ejectMedia() || d->forceNoEject )
+    jobFinished(d->success);
   else {
     emit newSubTask( i18n("Ejecting DVD") );
-    connect( K3bDevice::eject( burnDevice() ), 
+    connect( K3bDevice::eject( burnDevice() ),
 	     SIGNAL(finished(K3bDevice::DeviceHandler*)),
-	     this, 
+	     this,
 	     SLOT(slotEjectingFinished(K3bDevice::DeviceHandler*)) );
   }
 }
@@ -509,7 +576,8 @@ void K3bGrowisofsWriter::slotProcessExited( KProcess* p )
 
 void K3bGrowisofsWriter::slotRingBufferFinished( bool )
 {
-  if( d->canceled && !d->process->isRunning() ) {
+  if( !d->finished ) {
+    d->finished = true;
     // this will unblock and eject the drive and emit the finished/canceled signals
     K3bAbstractWriter::cancel();
   }
@@ -521,7 +589,7 @@ void K3bGrowisofsWriter::slotEjectingFinished( K3bDevice::DeviceHandler* dh )
   if( !dh->success() )
     emit infoMessage( i18n("Unable to eject media."), ERROR );
 
-  emit finished(d->success);
+  jobFinished(d->success);
 }
 
 
@@ -543,4 +611,16 @@ void K3bGrowisofsWriter::slotFlushingCache()
   }
 }
 
+
+void K3bGrowisofsWriter::setMultiSessionInfo( const QString& info )
+{
+  d->multiSessionInfo = info;
+}
+
+
+void K3bGrowisofsWriter::setForceNoEject( bool b )
+{
+    d->forceNoEject = b;
+}
+
 #include "k3bgrowisofswriter.moc"
