@@ -1,7 +1,7 @@
 
 /*
  *
- * $Id: main.cpp 642803 2007-03-15 12:44:58Z trueg $
+ * $Id: main.cpp 689578 2007-07-18 15:56:58Z trueg $
  *
  * This file is part of the K3b project.
  * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
@@ -69,6 +69,8 @@ static KCmdLineOptions options[] =
 	{ "lang <language>", I18N_NOOP("Set the GUI language"), 0 },
 	{ "nosplash", I18N_NOOP("Disable the splash screen"), 0 },
 	{ "ao <method>", I18N_NOOP("Set the audio output method (like arts or alsa depending on the installed plugins)"), 0 },
+        { "device <device>", I18N_NOOP( "Set the device to be used for new projects (This option has no effect. "
+                                        "Its main purpose is to enable handling of empty media from the KDE Media Manager)." ), 0 },
         KCmdLineLastOption
     };
 
